package ajedrez.server.comunicaciones;


import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

import ajedrez.client.comunicaciones.IJugador;


public interface IAjedrezServer extends Remote {
	
	/**
	 * Devuelve un Vector con los nombres de los jugadores conectados.
	 */
	public Vector<String> getJugadoresConectados() throws RemoteException;
	
	
	/**
	 * Registra al usuario en la base de datos.
	 */
	public boolean registrar(String sUsuario, String sContrasea) throws RemoteException;		
	
	
	/**
	 * Comprueba la identificacin del usuario en la base de datos.
	 * @returns   Devuelve '0' si la identificacin ha sido correcta, '-1' si
	 *            nicamente la contrasea est equivocada, '-2' si el usuario
	 *            no est registrado en el sistema y '-3' si el usuario ya se
	 *            encontraba identificado.
	 */
	public int identificar(String sUsuario, String sContrasea) throws RemoteException;
	
	
	/**
	 * Identifica al usuario en el sistema.
	 */
	public void identificar(String sUsuario, IJugador jugador) throws RemoteException;		
	
	
	/**
	 * Finaliza la sesin del usuario.
	 * @returns   '0' si se ha cerrado la sesin correctamente o '-1' si el
	 *            usuario no estaba identificado en el sistema.
	 */
	public int cerrarSesion(String sUsuario) throws RemoteException;
	
	
	/**
	 * Obtiene los datos clasificatorios del jugador.
	 * @return   Devuelve una matriz de cinco componentes:
	 *           0 - partidas jugadas
	 *           1 - partidas ganadas
	 *           2 - partidas perdidas
	 *           3 - partidas en tablas
	 *           4 - puntuacin
	 */
	public String[] getClasificacion(String sUsuario) throws RemoteException;
	
	
	/**
	 * Realizacin de un reto.
	 */
	public void retar(String sRetador, String sRetado) throws RemoteException;
	
	
	/**
	 * Recibe la contestacin del reto e informa al retador, creando la
	 * partida si es preciso.
	 */
	public void contestarReto(String sRetado, String sRetador, boolean bRespuesta) throws RemoteException;
	
	
	/**
	 * Gestiona la peticin de un movimiento.
	 * @returns   Clave indicativa del suceso producido. Puede ser:
	 *          0: el movimiento se ha realizado correctamente
	 *          1: estado 0 y el jugador actual ha eliminado una pieza del contrincante
	 *          2: estado 0 y la pieza es un pen que ha llegado al final del tablero
	 *          3: estados 1 y 2
	 *          4: estado 0 y el movimiento realizado ha sido tomar al paso
	 *          5: estado 0 y el movimiento realizado ha sido el enroque corto
	 *          6: estado 0 y el movimiento realizado ha sido el enroque largo
	 *         1_: estado menor de 10 y se produce jaque al rey contrario
	 *         2_: estado menor de 10 y el jugador actual ha ganado la partida
	 *         3_: estado menor de 10 y se han producido tablas
	 *         -1: no es el turno del jugador actual
	 *         -2: no hay ninguna pieza en el origen
	 *         -3: la pieza de la casilla origen no es del jugador actual
	 *         -4: hay una pieza del jugador actual en la casilla destino
	 *         -5: el movimiento no se corresponde con la pieza del origen
	 *         -6: hay una pieza en medio que impide el movimiento solicitado
	 *         -7: el movimiento dejara al rey en jaque
	 */
	public int mover(int nId, char cColor, String sCasillaOrigen, String sCasillaDestino) throws RemoteException;
	
	
	/**
	 * Gestiona la conversin de un pen que ha llegado al final del tablero.
	 * @returns   Clave indicativa del suceso producido. Puede ser:
	 *          0: conversin realizada sin incidencias
	 *          1: estado 0 y se produce jaque al rey contrario
	 *          2: estado 0 y el jugador actual ha ganado la partida
	 *          3: estado 0 y se han producido tablas
	 */
	public int convertirPeon(int nId, char cColor, char cTipo, int nFila, int nColumna) throws RemoteException;
	
	
	/**
	 * Un jugador ha propuesto tablas.
	 * @param nId     Identificador de la partida.
	 * @param cColor  Color del jugador.
	 */
	public void proponerTablas(int nId, char cColor) throws RemoteException;
	
	
	/**
	 * Recibe la contestacin de la propuesta de tablas e informa al jugador,
	 * finalizando la partida si es preciso.
	 * @param nId     Identificador de la partida.
	 * @param cColor  Color del jugador.
	 */
	public void contestarTablas(int nId, char cColor, boolean bRespuesta) throws RemoteException;
	
	
	/**
	 * Un jugador ha solicitado abandonar.
	 * @param nId     Identificador de la partida.
	 * @param cColor  Color del jugador.
	 */
	public void abandonar(int nId, char cColor) throws RemoteException;
	
} /* Fin interfaz IAjedrezServer */
